package egs;

import abp.*;
import util.*;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

public class RPS{
  static enum Move{
    ROCK, PAPER, SCISSORS;

    boolean beats(Move m){
      return this == ROCK && m == SCISSORS ||
              this == PAPER && m == ROCK ||
               this == SCISSORS && m == PAPER;
    }
  }

  abstract static class Player{
    abstract Move pickMove();
    abstract void recordOpponent(Move m);
  }

  static class ScriptedPlayer extends Player{
    private final Iterator<Move> moveIterator;
    private final Map<Move,Long> moveFreqs = new LinkedHashMap<Move,Long>();

    private ScriptedPlayer(Iterator<Move> mitr){
      moveIterator = mitr;
      for (Move m : Move.values())
        moveFreqs.put(m,(Long)0L);
    }

    Move pickMove(){
      Move m = moveIterator.next();
      moveFreqs.put(m, moveFreqs.get(m) + 1);
      return m;
    }

    void recordOpponent(Move m){ }

    public String toString(){
      return super.toString() + ":\n"+Util.histogramToString(moveFreqs);
    }
  }

  static class AdaptivePlayer extends Player{
    private Move lastMove;
    private final RowAdaptive<Move> adptv = new RowAdaptive<Move>(Move.values());

    Move pickMove(){
      return (lastMove = adptv.suggest());
    }

    void recordOpponent(Move m){
      if (lastMove.beats(m)) {
        adptv.reward(lastMove,1,0);
      } else if (m.beats(lastMove)) {
        adptv.reward(lastMove,-1,0);
      }
    }

    public String toString(){ return adptv.toString(); }
  }

  public static void main(String[] args){
    Player p1 = new ScriptedPlayer(
                  new Dist<Move>(Move.values(),
                                  new double[]{1,20,1}).iterator());
    Player p2 = new AdaptivePlayer();
    int p1w = 0, p2w = 0, ties = 0;
    for (int i = 0; i < 100000; i++) {
      Move m1 = p1.pickMove(),
           m2 = p2.pickMove();
//      System.out.printf("%12s vs. %12s%n", m1, m2);
      p1.recordOpponent(m2);
      p2.recordOpponent(m1);
      if (m1.beats(m2)) {
        p1w++;
      } else if (m2.beats(m1)) {
        p2w++;
      } else {
        ties++;
      }
      if (i % 10000 == 0)
        System.out.println(p2);
    }

    System.out.println(p1);
    System.out.println(p2);
    int gs = p1w + p2w + ties;
    System.out.printf("%d (%5.2f %%) / %d (%5.2f %%) / %d (%5.2f %%) %n",
      p1w, 100.0 * p1w / gs,
      p2w, 100.0 * p2w / gs,
      ties, 100.0 * ties / gs);

//    AdaptiveProcess<Move,Double> p = new AdaptiveProcess<Move,Double>();

/*
    Adaptive<Move> qvalue = new RAAdaptive<Move>(Move.values());
//    System.out.println(Move.values());
    System.out.println(qvalue.action());
    System.out.println(qvalue);
    qvalue.adapt(Move.ROCK,12);
    System.out.println(qvalue.action());
    System.out.println(qvalue);
    qvalue.adapt(Move.ROCK,-132);
    System.out.println(qvalue.action());
    System.out.println(qvalue);
    qvalue.adapt(Move.PAPER,2);
    System.out.println(qvalue.action());
    System.out.println(qvalue);
*/
  }
}

